/******************************************************************************
*******************************************************************************
**                                                                           **
**      C Swing - A graphical file and directory maintenance utility         **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: S. J. Brown                                    **
**                                                                           **
**      Modifications by many others; see the documentation for a list       **
**                                                                           **
**      Copyright 1994, Harry Flowers and Foteos Macrides                    **
**                                                                           **
**      Terms of Copyright:  This software may be freely distributed         **
**          and used by anyone, providing the distributor provides it        **
**          free of charge or at a price which only covers distribution      **
**          costs.  It may not be sold.  Note that versions prior to         **
**          V3.7.4 were public domain, and are not subject to these terms.   **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
 
/*
**  swing_3
*/
#include <stdio.h>
#include <smgdef.h>
#include <smgmsg.h>
#include <ssdef.h>
#include <time.h>
#include "more_descrip.h"
#include "swing.h"
#include "swing_refs.h"

#define SIZEOF_TIME	12

static char old_sub_title[SCREEN_WIDTH*2 + 1];
static char old_clock_time[SIZEOF_TIME+1];
static int old_effect = -1;
static char *month[] = {"Jan","Feb","Mar","Apr","May","Jun",
                        "Jul","Aug","Sep","Oct","Nov","Dec"};

static int big_message_on = FALSE;
static int big_message_inited = FALSE;
static int display_bm;

#define BIG_MESSAGE_ROWS 	5
#define BIG_MESSAGE_COLUMNS	66
#define BIG_MESSAGE_X		8
#define BIG_MESSAGE_Y		(page_length-BIG_MESSAGE_ROWS)

redo_display(option)
int option;
{
    int scroll_heave_up, scroll_heave_down;

    if (option)
    {
        scroll_heave_up = (3 * DISPLAY1_ROWS) / 4;
        scroll_heave_down = DISPLAY1_ROWS / 4;
    }
    else
	scroll_heave_up = scroll_heave_down = 0;

    /*
    **  make sure that we are now displaying current_line current_level 
    **  on the screen somewhere
    */
    if (( current_line >= current_top_line && 
          current_line < current_top_line + DISPLAY1_ROWS) ||
          total_lines <= DISPLAY1_ROWS )
    {
       /*
       **  we don't need to do anything here
       */
       return(TRUE);
    }

    if (current_line > current_top_line + DISPLAY1_ROWS - 1)
    {
        while( (current_line > (current_top_line + scroll_heave_down)) &&
              ((current_top_line + DISPLAY1_ROWS) < total_lines))
        {
            current_top_line++;
        }

        refresh_display();
        return(TRUE);
    }

    if (current_line < current_top_line)
    {
        while (( current_line < (current_top_line + scroll_heave_up)) &&
                current_top_line)
        {
            current_top_line--;
        }

        refresh_display();
        return(TRUE);
    }
}

big_message_init()
{
    int rows, columns, i;

    if (big_message_inited)
        return(TRUE);

    big_message_inited = TRUE;

    rows    = BIG_MESSAGE_ROWS;
    columns = BIG_MESSAGE_COLUMNS;
    i = smg$create_virtual_display(&rows,
                                   &columns,
                                   &display_bm,
                                   &SMG$M_BORDER);
    if (i != SS$_NORMAL)
        panic(i,"create virtual display displayf","big message init");
}

big_display_on()
{
    int y;

    y = BIG_MESSAGE_Y;

    big_message_init();
    smg$erase_display(&display_bm);
    if (!big_message_on)
        smg$paste_virtual_display(&display_bm,
                                  &paste,
                                  &y,
                                  &BIG_MESSAGE_X);
    smg$end_display_update(&display_bm);

    big_message_on = TRUE;
}

big_display_off()
{
    if (big_display_on)
    {
        big_message_on = FALSE;
        smg$unpaste_virtual_display(&display_bm,
                                    &paste);
    }
}

message(text)
char *text;
{
    if (text[0])
    {
        message_set = TRUE;
        if (strlen(text) > SCREEN_WIDTH)
        {
            /*
            **  message too big for the bottom line, so we'll use a special
            **  display
            */
            big_display_on();

            /*
            **  make sure no bottom line message still there
            */
            smg$erase_display(&displayb);

            /*
            **  now send the output to the big display
            */
            smg$set_cursor_abs(&display_bm,
                               &1,
                               &1);

            smg$put_line(&display_bm,
                         DSC1(text),
                         0,
                         0,
                         0,
                         &SMG$M_WRAP_WORD);
            rest_cursor();
        }
        else
        {
            /*
            **  put message on the bottom line
            */
            big_display_off();
            smg$begin_display_update(&displayb);
            smg$set_cursor_abs(&displayb,&1,&1);
            smg$put_line(&displayb,DSC1(text));
            rest_cursor();
            smg$end_display_update(&displayb);
        }
    }
    else
    {
        /*
        **  no text, so clear the message display
        */
        message_set = FALSE;
        smg$erase_display(&displayb);
        big_display_off();
    }
}

title(text)
char *text;
{
    /*
    **  display a title on the first line of window displayt
    */
    smg$set_cursor_abs(&displayt,&1,&1);
    smg$put_line(&displayt,DSC1(text));
}

sub_title_normal(text)
char *text;
{
    /*
    **  no special effects
    */
    put_sub_title(text,0);
}

sub_title_underline(text)
char *text;
{
    /*
    **  no special effects
    */
    put_sub_title(text,SMG$M_UNDERLINE);
}

update_clock()
{
    /*
    **  force a new clock time
    */
    put_sub_title(old_sub_title,old_effect);
}

put_sub_title(text,effect)
char *text;
int effect;
{
    /*
    **  display a sub_title on the second line of window displayt
    **
    **  to save time we'll check what's on display
    */
    char line[SCREEN_WIDTH+1];
    char clock_time[SIZEOF_TIME+1];
    char me[41];
    int i;

    get_time(clock_time);

    if (strcmp(text,old_sub_title) == 0 && old_effect == effect &&
        strcmp(clock_time,old_clock_time) == 0)
        return(TRUE);
    else
    {
        strcpy(old_clock_time,clock_time);
        old_effect = effect;
        strcpy(old_sub_title,text);
        strcpy(me,this_user);
    }

    sprintf(line,"%.*s",SCREEN_WIDTH,text);
    for (i = strlen(line); i < SCREEN_WIDTH; i++)
        line[i] = ' ';
    line[SCREEN_WIDTH] = 0;

    /*
    **  look see if we've room for the user on the left
    */
    for (i = 0; i < strlen(me) + 2; i++)
    {
        if (line[i] != ' ')
            goto no_me;
    }

    /*
    **  tack me in
    */
    for (i = 0; i < strlen(me); i++)
    {
        line[i] = me[i];
    }

    
no_me:;
    /*
    **  if the last SIZEOF_TIME+1 chars are spaces, 
    **  we'll super-impose the time over the last SIZEOF_TIME chars
    */
    if (strncmp("              ",
                &line[SCREEN_WIDTH-(SIZEOF_TIME+1)],
                (SIZEOF_TIME+1)) == 0)
        strcpy(&line[SCREEN_WIDTH-SIZEOF_TIME],clock_time);

    smg$begin_pasteboard_update(&paste);
    smg$set_cursor_abs(&displayt,&2,&1);
    smg$put_line(&displayt,DSC1(line),&0,&effect);
    smg$end_pasteboard_update(&paste);
}

get_time(the_time)
char *the_time;
{
    int secs;
    struct tm *tt;

    secs = time(0);
    tt = localtime(&secs);

    sprintf(the_time,"%02d %.3s %02d:%02d",
            tt->tm_mday,
            month[tt->tm_mon],
            tt->tm_hour, 
            tt->tm_min); 
}
